// App.h: interface for the CApp class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(APP_H_INCLUDED)
#define APP_H_INCLUDED

#pragma once

#include "OpenGL\\3DCamera.h"

typedef enum
{
	HA_NONE,
	HA_MOVE,
	HA_ROTATE,
	HA_ZOOM,
} HAND_ACTION;

/////////////////////////////////////////////////////////////////////////////
// CApp
// Application base class

class CApp
{
protected:
	HINSTANCE m_hInstance;// Handle to application instance
	HACCEL m_hAccelTable;	// Accelerator table
	HWND m_hWnd;			// Handle to window

	HDC m_hDC;				// Handle to DC
	HGLRC m_hRC;		  // Handle to RC

	HAND_ACTION m_hand_action;
	C3dCamera   m_Camera;   // Our windows camera
	int         m_capture_x;
	int         m_capture_y;

	virtual void DrawScene() = 0;
	virtual void KillScene() = 0;
	virtual void InitScene() = 0;
	
	virtual BOOL OnCommand(int nCmdID, int nEvent);	
	void OnCreate();
	void OnDestroy();
	void OnSize(int cx, int cy);
	void OnLButtonDown(int, int);
	void OnMouseMove(int, int);
	void OnLButtonUp(int, int);
	virtual void OnPaint(HDC hDC);
	virtual void PreCreateWindow(WNDCLASSEX &wcex, DWORD &dwStyle, DWORD &dwExStyle, int &x, int &y, int &cx, int &cy);
	BOOL SetWindowPixelFormat(HDC hDC);
	virtual BOOL InitInstance(HINSTANCE hInstance, int nCmdShow, LPSTR lpCmdLine);
	virtual void ExitInstance();

	void  FitToScene();
	void  RotateCamera(SIZE sdvig);
	void  TranslateCamera(SG_VECTOR sdvig);
	void  ZoomCamera(SIZE sdvig);

public:
	static CApp* m_pAppInstance;

	int Run(HINSTANCE hInstance, int nCmdShow, LPSTR lpCmdLine);
	virtual LRESULT WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);

	CApp();
	virtual ~CApp();
};

#endif // !defined(APP_H_INCLUDED)
